from nonebot.adapters.onebot.v11 import GroupMessageEvent
from dicergirl.common.const import LOGGERS_CACHE_FILE
from dicergirl.utils.handlers import get_group_id
from loguru._logger import Logger
from typing import Dict, List

import json


loggers: Dict[str, Dict[int, List[Logger | str]]] = {}
""" 正在运行的日志 """
saved_loggers: Dict[str, dict]
""" 存储的日志 """


def load_loggers() -> Dict[str, list]:
    """ 加载所有的已存储的日志 """
    return json.loads(open(LOGGERS_CACHE_FILE, "r").read())


def get_loggers(event) -> List[str]:
    """ 获取`event`所指向的群聊中所有的日志 """
    got_loggers = json.load(open(LOGGERS_CACHE_FILE, "r"))
    if not get_group_id(event) in got_loggers:
        return []

    return got_loggers[get_group_id(event)]


def add_logger(event: GroupMessageEvent, logname) -> bool:
    """ 新增日志序列 """
    global saved_loggers
    if not get_group_id(event) in saved_loggers.keys():
        saved_loggers[get_group_id(event)] = []

    try:
        saved_loggers[get_group_id(event)].append(logname)
        json.dump(saved_loggers, open(LOGGERS_CACHE_FILE, "w"))
        return True
    except:
        return False


def remove_logger(event: GroupMessageEvent, id: int) -> Dict[str, list]:
    """ 从存储的`loggers.json`中移除指定`logger` """
    saved_loggers[get_group_id(event)].pop(id)
    json.dump(saved_loggers, open(LOGGERS_CACHE_FILE, "w"))
    return saved_loggers


from pathlib import Path
from datetime import datetime
from multilogging import multilogger

from nonebot.matcher import Matcher
from nonebot.plugin import on, on_request, on_notice, PluginMetadata
from nonebot.adapters import Bot as Bot
from nonebot.adapters.onebot import V11Bot
from nonebot.adapters.onebot.v11 import MessageEvent, GroupMessageEvent, Event, MessageSegment
from nonebot.adapters.onebot.v11.event import FriendRequestEvent, GroupRequestEvent, GroupDecreaseNoticeEvent
from nonebot.adapters.onebot.v11.exception import ActionFailed
from nonebot.internal.matcher.matcher import Matcher

from .handlers.on import on_startswith
from .handlers.general import show_handler, set_handler, del_handler, roll, shoot

from .reply.manager import manager

from .utils.operator import botoff, boton, get_name, get_status, init, set_name
from .utils.handlers import get_friend_qids, get_group_id, get_mentions, get_user_card
from .utils.formatters import format_msg, format_str
from .utils.loggers import add_logger, get_loggers, remove_logger, loggers
from .utils.admin import add_super_user, get_super_users, get_uuid, is_super_user, make_uuid, rm_super_user
from .utils.modes import get_mode, set_mode
from .utils.role import rolekp, roleob
from .utils.version import run_shell_command, get_latest_version
from .utils.plugins import modes
from .utils.parser import CommandParser, Commands, Only, Optional, Required, Positional
from .utils.cards import Cards
from .utils.charactors import Character
from .utils.blacklist import blacklist
from .utils.update import require_update
from .utils.settings import DEBUG, debugon, debugoff

from .plugins.parse import get_plugins
from .plugins.operation import install, remove, upgrade as plgupgrade

from .common.exceptions.pluginerror import PluginExistsError, PluginInstallFailedError, PluginNotFoundError, PluginUninstallFailedError
from .common.messages import help_message
from .common.registers import regist_all
from .common.const import DICERGIRL_LOGS_PATH, VERSION

import logging
import sys
import platform
import psutil
import html
import nonebot
import re
import json
import asyncio


__version__ = VERSION
__plugin_meta__ = PluginMetadata(
    name="DicerGirl",
    description="新一代跨平台开源 TRPG 骰娘框架",
    usage="开箱即用",
    type="application",
    homepage="https://gitee.com/unvisitor/dicer",
    supported_adapters={"~onebot.v11"},
)
__author__ = "苏向夜 <fu050409@163.com>"

logger = multilogger(name="DicerGirl", payload="main")
current_dir = Path(__file__).resolve().parent


try:
    driver = nonebot.get_driver()
    initalized = True
except ValueError:
    initalized = False

if initalized:
    # 开发者指令
    testcommand = on_startswith(".test", priority=1, block=True)
    debugcommand = on_startswith(".debug", priority=1, block=True)

    # 消息监视及日志指令
    messagemonitor = on("message", priority=2, block=False)
    dgmessagemonitor = on("message_sent", priority=2, block=False)
    logcommand = on_startswith(".log", priority=2, block=True)

    # 请求及事件处理
    addrequest = on_request(priority=2, block=True)
    kickedevent = on_notice(priority=2, block=False)

    # 机器人管理指令
    superusercommand = on_startswith((".su", ".sudo"), priority=2, block=True)
    botcommand = on_startswith(".bot", priority=2, block=True)
    dismisscommand = on_startswith((".dismiss", ".exit", ".leave", ".bye"), priority=2, block=True)
    registcommand = on_startswith((".regist", ".reg"), priority=2, block=True)
    versioncommand = on_startswith((".version", ".v"), priority=2, block=True)

    # 基本指令
    helpcommand = on_startswith((".help", ".h"), priority=3, block=True)
    rollcommand = on_startswith((".r", ".roll"), priority=4, block=True)
    racommand = on_startswith(".ra", priority=3, block=True)
    rhcommand = on_startswith(".rh", priority=3, block=True)
    rahcommand = on_startswith(".rah", priority=3, block=True)
    delcommand = on_startswith((".del", ".delete"), priority=3, block=True)
    showcommand = on_startswith((".show", ".display"), priority=3, block=True)
    setcommand = on_startswith((".set", ".st", ".s"), priority=4, block=True)
    modecommand = on_startswith((".mode", ".m"), priority=3, block=True)
    shootcommand = on_startswith((".sht", ".shoot"), priority=3, block=True)
    attackcommand = on_startswith((".at", ".attack"), priority=3, block=True)
    damcommand = on_startswith((".dam", ".damage"), priority=3, block=True)
    encommand = on_startswith((".en", ".encourage"), priority=3, block=True)

    # 身份组指令
    rolecommand = on_startswith(".role", priority=3, block=True)
    rolekpcommand = on_startswith(".kp", priority=3, block=True)
    roleobcommand = on_startswith(".ob", priority=3, block=True)

    # 定时任务
    scheduler = nonebot.require("nonebot_plugin_apscheduler").scheduler

    @driver.on_startup
    async def _() -> None:
        """ `Nonebot2`核心加载完成后的初始化方法 """
        global DEBUG
        logger.info(f"DicerGirl 版本 {VERSION} 初始化中...")
        if DEBUG:
            logging.getLogger().setLevel(logging.DEBUG)
            logger.remove()
            logger.add(
                sys.stdout,
                level = "DEBUG"
            )
            logger.info("DEBUG 模式已启动.")
        regist_all()
        init()
        logger.success("DicerGirl 初始化完毕.")

    @addrequest.handle()
    async def friendaddapproval(bot: V11Bot, event: FriendRequestEvent):
        if event.user_id in await get_friend_qids(bot):
            return

        if event.get_user_id() in blacklist.get_blacklist():
            try:
                await event.reject(bot)
            except ActionFailed:
                return

            return await bot.send_private_msg(
                user_id=event.user_id, # TODO
                message=manager.process_generic_event(
                    "main.friend.reject",
                    event=event
                )
            )

        try:
            await event.approve(bot)
        except ActionFailed:
            return

        super_users = get_super_users()
        for superuser in super_users:
            try:
                await bot.send_private_msg(
                    user_id=superuser,
                    message=manager.process_generic_event(
                            "main.friend.approve",
                            event=event
                        )
                    )
            except ActionFailed:
                pass

        await asyncio.sleep(5.5)

        return await bot.send_private_msg(
            user_id=event.user_id,
            message=manager.process_generic_event(
                "main.friend.new",
                event=event
            )
        )

    @addrequest.handle()
    async def groupaddapproval(bot: V11Bot, event: GroupRequestEvent):
        if event.sub_type == "add":
            return

        if event.get_user_id() in blacklist.get_blacklist() or event.group_id in blacklist.get_group_blacklist():
            try:
                await event.reject(bot)
            except ActionFailed:
                await bot.set_group_leave(
                    group_id=event.group_id
                )

            return await bot.send_private_msg(
                user_id=event.user_id,
                message=manager.process_generic_event(
                    "GroupForbidden",
                    event=event
                )
            )

        try:
            await event.approve(bot)
        except ActionFailed:
            return

        super_users = get_super_users()
        for superuser in super_users:
            await bot.send_private_msg(
                user_id=superuser,
                message=manager.process_generic_event(
                        "GroupApproval",
                        event=event
                    )
                )

        if not super_users:
            await bot.send_private_msg(
                user_id=event.self_id,
                message=manager.process_generic_event(
                    "GroupApproval",
                    event=event
                )
            )

    @kickedevent.handle()
    async def onkickhandler(bot: V11Bot, event: GroupDecreaseNoticeEvent):
        if not event.is_tome():
            logger.info(f"用户[{event.user_id}]被移出群聊.")
            return

        if not event.group_id and event.operator_id:
            return

        if event.group_id:
            blacklist.add_group_blacklist(str(event.group_id))

        if event.operator_id:
            blacklist.add_blacklist(str(event.operator_id))

        try:
            await bot.call_api(
                "delete_friend",
                user_id = event.operator_id
            )
        except ActionFailed:
            return

        super_users = get_super_users()
        for superuser in super_users:
            await bot.send_private_msg(
                user_id=superuser,
                message=manager.process_generic_event(
                        "BlacklistAdded",
                        event=event,
                        UserID=event.operator_id
                    )
                )

        if not super_users:
            await bot.send_private_msg(
                user_id=event.self_id,
                message=manager.process_generic_event(
                    "BlacklistAdded",
                    event=event,
                    UserID=event.operator_id
                )
            )

    @testcommand.handle()
    async def testhandler(bot: V11Bot, matcher: Matcher, event: MessageEvent):
        """ 测试指令 """
        args = format_msg(event.get_message(), begin=".test")
        cp = CommandParser(
            Commands([
                Only("all"),
                Only("split"),
                Only("invite"),
                Only("at"),
                Only("userid"),
                Optional("run", str)
            ]),
            args=args,
            auto=True
        )

        if not is_super_user(event):
            return await matcher.send(manager.process_generic_event(
                "PermissionDenied",
                event=event
            ))

        reply = ""
        if cp.nothing or cp.results["all"]:
            import json
            reply += f"收到消息: {event.get_message()}\n"
            reply += f"消息来源: {event.group_id} - {event.get_user_id()}\n"
            reply += f"消息拆析: {args}\n"
            reply += f"指令拆析: {cp.results}\n"
            reply += f"消息原始内容: {event.get_plaintext()}\n"
            reply += f"消息json数据: {event.json()}\n"
            reply += f"消息发送者json信息: {json.loads(event.json())['sender']}\n"
            reply += f"发送者昵称: {json.loads(event.json())['sender']['nickname']}"
            return await matcher.send(reply)
        else:
            cp = cp.results

        if cp["invite"]:
            await matcher.send("http://qm.qq.com/cgi-bin/qm/qr?_wv=1027&k=hvaf8JGmEXA3N9r4SGgpghDti31aW1bR&authKey=%2Bux%2BedOIguriMYBMGe40coeOT7mx%2B99%2FVMbK0MvE2w1AsVQLLK%2B0hBO6vVB%2Bmlws&noverify=0&group_code=770386358")
            return await bot.send_private_msg(
                user_id=event.user_id,
                message="""[CQ:json,data={"app":"com.tencent.qun.invite"&#44;"config":{"autosize":0&#44;"ctime":1696238280&#44;"round":1&#44;"token":"c05ba4b99859234b1f2b225b74740808"&#44;"type":"normal"}&#44;"meta":{"news":{"desc":"邀请你加入群聊“DicerGirl 公测群 | 未知访客”，进入可查看详情。"&#44;"jumpUrl":"mqqapi://group/invite_join?src_type=internal&amp;version=1&amp;groupcode=770386358&amp;msgseq=1696238280521100&amp;groupname=DicerGirl+%e5%85%ac%e6%b5%8b%e7%be%a4+%7c+%e6%9c%aa%e7%9f%a5%e8%ae%bf%e5%ae%a2&amp;senderuin=1264983312&amp;receiveruin=1557089913"&#44;"preview":"https://p.qlogo.cn/gh/770386358/770386358/?t=1696238280"&#44;"tag":"邀请加群"&#44;"tagIcon":"https://downv6.qq.com/innovate/group_ark_icon.png"&#44;"title":"邀请你加入群聊"}}&#44;"prompt":"邀请加群"&#44;"ver":"1.0.0.31"&#44;"view":"news"}]""",
                auto_escape=False
            )

        if cp["at"]:
            return await matcher.send(MessageSegment.at(user_id=event.user_id))

        if cp["userid"]:
            return await matcher.send(
                manager.process_generic_event(
                    "main.test.userid",
                    event=event
                )
            )

        if cp["run"]:
            try:
                result = eval(cp["run"])
            except Exception as e:
                result = f"出现异常：{e}"

            return await matcher.send(result)

    @debugcommand.handle()
    async def debughandler(matcher: Matcher, event: MessageEvent):
        """ 漏洞检测指令 """
        args = format_msg(event.get_message(), begin=".debug")
        if not is_super_user(event):
            await matcher.send(manager.process_generic_event(
                "PermissionDenied",
                event=event
            ))
            return

        if args:
            logger.debug(args)
            if args[0] == "off":
                debugoff()
                logging.getLogger().setLevel(logging.INFO)
                logger.remove()
                logger.add(
                    sys.stdout,
                    level = "INFO"
                )
                logger.info("输出等级设置为 INFO.")
                return await matcher.send(manager.process_generic_event(
                    "DebugOff",
                    event=event
                ))
        else:
            debugon()
            logging.getLogger().setLevel(logging.DEBUG)
            logger.remove()
            logger.add(
                sys.stdout,
                level = "INFO"
            )
            logger.info("输出等级设置为 DEBUG.")
            return await matcher.send(manager.process_generic_event(
                "DebugOn",
                event=event
            ))

        if args[0] == "on":
            debugon()
            logging.getLogger().setLevel(logging.DEBUG)
            logger.remove()
            logger.add(
                sys.stdout,
                level = "INFO"
            )
            logger.info("输出等级设置为 DEBUG.")
            return await matcher.send(manager.process_generic_event(
                "DebugOn",
                event=event
            ))
        else:
            return await matcher.send("错误, 我无法解析你的指令.")

    @superusercommand.handle()
    async def superuser_handler(matcher: Matcher, event: MessageEvent):
        """ 超级用户管理指令 """
        args = format_str(event.get_message(), begin=(".su", ".sudo"))
        arg = list(filter(None, args.split(" ")))

        if len(arg) >= 1:
            if arg[0].lower() == "exit":
                if not rm_super_user(event):
                    return await matcher.send(manager.process_generic_event(
                        "NotManagerYet",
                        event=event
                    ))

                return await matcher.send(manager.process_generic_event(
                    "ManagerExit",
                    event=event
                ))

        if is_super_user(event):
            return await matcher.send(manager.process_generic_event(
                "AlreadyManager",
                event=event
            ))

        if not args:
            logger.critical(f"超级令牌: {make_uuid()}")
            await matcher.send(manager.process_generic_event(
                "AuthenticateStarted",
                event=event
            ))
        else:
            if not args == get_uuid():
                await matcher.send(manager.process_generic_event(
                    "AuthenticateFailed",
                    event=event
                ))
            else:
                add_super_user(event)
                await matcher.send(manager.process_generic_event(
                    "AuthenticateSuccess",
                    event=event
                ))

    @botcommand.handle()
    async def bothandler(bot: V11Bot, matcher: Matcher, event: MessageEvent, args: list=[]):
        """ 机器人管理指令 """
        if get_mentions(event) and not event.to_me:
            return

        args = format_msg(event.get_message(), begin=".bot", zh_en=True) if not args else args
        commands = CommandParser(
            Commands([
                Only(("version", "v", "bot", "版本")),
                Only(("exit", "bye", "leave", "离开")),
                Only(("on", "run", "start", "启动")),
                Only(("off", "down", "shutdown", "关闭")),
                Only(("upgrade", "up", "更新")),
                Optional(("downgrade", "降级"), str),
                Optional(("name", "命名"), str),
                Only(("status", "状态")),
                Optional(("plgup", "pluginup", "升级"), str),
                Optional(("install", "add", "安装"), str),
                Optional(("remove", "del", "rm", "删除", "卸载"), str),
                Only(("mode", "list", "已安装")),
                Only(("store", "plugins", "商店")),
                Optional(("search", "搜索"), str),
                Positional("first", str)
            ]),
            args=args,
            auto=True
        )

        if commands.nothing or commands.results["version"]:
            return await versionhandler(matcher=matcher, event=event)
        else:
            commands = commands.results

        if commands["exit"]:
            logger.info(f"{get_name()}退出群聊: {event.group_id}")
            await matcher.send(manager.process_generic_event(
                "GroupLeaveSet",
                event=event
            ))
            return await bot.set_group_leave(group_id=event.group_id)

        if commands["on"]:
            boton(event)
            return await matcher.send(manager.process_generic_event(
                "BotOn",
                event=event
            ))

        if commands["off"]:
            botoff(event)
            return await matcher.send(manager.process_generic_event(
                "BotOff",
                event=event
            ))

        if commands["status"]:
            try:
                system = platform.freedesktop_os_release()["PRETTY_NAME"]
            except:
                system = platform.platform()

            memi = psutil.Process().memory_info()
            rss = memi.rss / 1024 / 1024
            total = psutil.virtual_memory().total / 1024 / 1024

            reply = f"DicerGirl {VERSION}, {'正常运行' if get_status(event) else '指令限制'}\n"
            reply += f"骰娘尊名: {get_name()}\n"
            reply += f"操作系统: {system}\n"
            reply += f"CPU 核心: {psutil.cpu_count()} 核心\n"
            reply += f"Python 版本: {platform.python_version()}\n"
            reply += "系统内存占用: %.2fMB/%.2fMB\n" % (rss, total)
            reply += f"漏洞检测模式: {'on' if DEBUG else 'off'}"
            return await matcher.send(reply)

        if commands["upgrade"]:
            await matcher.send("检查版本更新中...")
            newest_version = await get_latest_version("dicergirl")

            if require_update(VERSION, newest_version):
                if {"a", "b"} & set(newest_version) and commands["first"] in ("force", "强制"):
                    return await matcher.send(f"发现新版本 dicergirl {newest_version}, 然而该版本为{'公测' if 'b' in newest_version else '内测'}版本.\n使用`.bot upgrade force`强制更新到该版本.")

                await matcher.send(f"发现新版本 dicergirl {newest_version}, 开始更新...")
                upgrade = await run_shell_command(f"\"{sys.executable}\" -m pip install dicergirl -i https://pypi.org/simple --upgrade")

                if upgrade["returncode"] != 0:
                    logger.error(upgrade['stderr'])
                    return await matcher.send("更新失败! 请查看终端输出以获取错误信息, 或者你可以再次尝试.")

                return await matcher.send(f"{get_name()}骰娘已更新为版本 {newest_version}.")

            return await matcher.send(f"我已经是最新版本的{get_name()}了!")

        if commands["downgrade"]:
            if not is_super_user(event):
                return await matcher.send("你没有管理员权限, 请先执行`.su`开启权限鉴定.")

            await matcher.send("警告! 执行降级之后可能导致无法再次自动升级!")
            await run_shell_command(f"\"{sys.executable}\" -m pip install dicergirl=={commands['downgrade']} -i https://pypi.org/simple")
            await matcher.send(f"{get_name()}已降级到 {commands['downgrade']}.")
            return

        if commands["name"]:
            if not is_super_user(event):
                return await matcher.send("你没有管理员权限, 请先执行`.su`开启权限鉴定.")

            sn = set_name(commands["name"])

            if isinstance(sn, bool):
                await bot.call_api(
                    "set_qq_profile",
                    **{
                        "nickname": commands["name"],
                    }
                )
                await matcher.send(manager.process_generic_event(
                    "NameSet",
                    event=event,
                    NewName=commands["name"]
                ))
            elif isinstance(sn, str):
                await matcher.send(sn)

            return

        if commands["mode"]:
            reply = "当前已正确安装的跑团插件:\n"
            for plugin in modes.keys():
                reply += f"{plugin.upper()} 模式:\n"
                if hasattr(modes[plugin], "__description__"):
                    reply += f"\t简介: {modes[plugin].__description__}\n"

                reply += f"\t切换: .mode {plugin}\n"

            reply.strip("\n")
            reply += f"当前的跑团模式为 {get_mode(event).upper()}."
            return await matcher.send(reply)

        if commands["store"]:
            official, community = await get_plugins()
            reply = "从商店搜索到以下插件:\n"
            reply += "Unvisitor 官方插件:\n"

            i = 1
            for name, detail in official.items():
                reply += f"  {i}. {detail['name']}[安装: .bot install {name}]\n"
                i += 1

            reply += "社区提供插件:\n"
            i = 1
            for name, detail in community.items():
                reply += f"  {i}. {detail['name']}[安装: .bot install {name}]\n"

            reply.rstrip("\n")
            return await matcher.send(reply)

        if commands["install"]:
            if commands["install"] in modes.keys():
                return await matcher.send(f"模块 {commands['install']} 已经安装了!")

            ins = await install(commands["install"])

            if ins is PluginNotFoundError:
                return await matcher.send(f"包 {commands['install']} 不存在.")
            elif ins is PluginInstallFailedError:
                return await matcher.send(f"包 {commands['install']} 安装失败.")
            elif ins == True:
                return await matcher.send(f"模块 {commands['install']} 安装完毕.")

            return

        if commands["remove"]:
            if not is_super_user(event):
                return await matcher.send("你没有管理员权限, 请先执行`.su`开启权限鉴定.")

            uns = await remove(commands["remove"])

            if uns is PluginUninstallFailedError:
                return await matcher.send(manager.process_generic_event(
                    "UninstallFailed",
                    event=event
                ))

            return await matcher.send(f"模块 {commands['remove']} 卸载完毕.")

        if commands["plgup"]:
            await matcher.send(f"检查插件 {commands['plgup']} 更新中...")
            if commands['plgup'] not in modes.keys():
                plg_version = None
            else:
                plg_version = modes[commands['plgup']].__version__

            if await get_latest_version(f"dicergirl-plugin-{commands['plgup']}") == plg_version:
                return await matcher.send(f"插件 {commands['plgup']} 已经是最新版本了.\n`.bot remove {commands['plgup']}`并`.bot install {commands['plgup']}`可以重新安装插件.\n如果遇到问题, 使用`.help 支持`获得开发者联系方式.")

            up = await plgupgrade(commands["plgup"])

            if up is PluginNotFoundError:
                return await matcher.send(f"包 {commands['plgup']} 似乎不存在?")
            elif up is PluginInstallFailedError:
                return await matcher.send(f"包 {commands['plgup']} 更新失败了.")
            elif up == True:
                return await matcher.send(f"插件 {commands['plgup']} 更新完毕.")

        return await matcher.send("未知的指令, 使用`.help bot`获得机器人管理指令使用帮助.")

    @dismisscommand.handle()
    async def dismisshandler(bot: V11Bot, matcher: Matcher, event: Event):
        await bothandler(bot=bot, matcher=matcher, event=event, args=["exit"])

    @logcommand.handle()
    async def loghandler(bot: V11Bot, matcher: Matcher, event: Event):
        """ 日志管理指令 """
        if not get_status(event) and not event.to_me:
            return

        args = format_msg(event.get_message(), begin=".log")
        commands = CommandParser(
            Commands([
                Only("show"),
                Only(("add", "new")),
                Optional("name", str),
                Optional("stop", int),
                Optional("start", int),
                Optional(("remove", "rm"), int),
                Optional(("download", "load"), int)
                ]),
            args=args,
            auto=True
            ).results

        if commands["show"]:
            gl = get_loggers(event)
            if len(gl) == 0:
                return await matcher.send("暂无存储的日志.")

            if not get_group_id(event) in loggers.keys():
                running = []
            else:
                running = loggers[get_group_id(event)].keys()

            reply = "该群聊所有日志:\n"
            for l in gl:
                index = gl.index(l)
                reply += f"序列 {index} : {l} : {'记录中' if index in running else '已关闭'}\n"
            reply.strip("\n")

            return await matcher.send(reply)

        if commands["download"] or commands["download"] == 0:
            gl = get_loggers(event)
            if commands["download"] > len(gl)-1:
                return await matcher.send(f"目标日志序列 {commands['download']} 不存在.")

            path = Path(gl[commands["download"]])
            return await bot.call_api(
                "upload_group_file",
                **{
                    "group_id": get_group_id(event),
                    "file": str(path),
                    "name": path.name
                }
            )

        if commands["add"]:
            if commands["name"]:
                logname = str(DICERGIRL_LOGS_PATH/(commands["name"]+".trpg.log"))
            else:
                logname = str(DICERGIRL_LOGS_PATH/(datetime.now().strftime("%Y-%m-%d-%H-%M-%S")+".trpg.log"))

            new_logger = multilogger(name="DG Msg Logger", payload="TRPG")
            new_logger.remove()
            new_logger.add(logname, encoding='utf-8', format="{message}", level="INFO")
            if not get_group_id(event) in loggers.keys():
                loggers[get_group_id(event)] = {}

            index = len(get_loggers(event))
            loggers[get_group_id(event)][index] = [new_logger, logname]

            if not add_logger(event, logname):
                raise IOError("无法新增日志.")

            return await matcher.send(f"新增日志序列: {index}\n日志文件: {logname}")

        if commands["stop"] or commands["stop"] == 0:
            if not get_group_id(event) in loggers.keys():
                await matcher.send("该群组无正在运行中的日志.")
                loggers[get_group_id(event)] = {}
                return

            if not commands["stop"] in loggers[get_group_id(event)]:
                await matcher.send("目标日志不存在.")
                return

            loggers[get_group_id(event)][commands["stop"]][0].remove()
            loggers[get_group_id(event)].pop(commands["stop"])
            await matcher.send(f"日志序列 {commands['stop']} 已停止记录.")
            return

        if commands["start"] or commands["start"] == 0:
            gl = get_loggers(event)
            if commands["start"] > len(gl)-1:
                await matcher.send(f"目标日志序列 {commands['start']} 不存在.")
                return

            logname = gl[commands["start"]]
            new_logger = multilogger(name="DG Msg Logger", payload="TRPG")
            new_logger.remove()
            new_logger.add(logname, encoding='utf-8', format="{message}", level="INFO")
            if not get_group_id(event) in loggers.keys():
                loggers[get_group_id(event)] = {}

            loggers[get_group_id(event)][commands["start"]] = [new_logger, logname]
            await matcher.send(f"日志序列 {commands['start']} 重新启动.")
            return

        if commands["remove"] or commands["remove"] == 0:
            gl = get_loggers(event)
            if not gl:
                await matcher.send("该群组从未设置过日志.")
                loggers[get_group_id(event)] = {}
                return

            if commands["remove"] > len(gl)-1:
                return await matcher.send(f"目标日志序列 {commands['remove']} 不存在.")

            index = len(gl)
            if get_group_id(event) in loggers.keys():
                if commands["remove"] in loggers[get_group_id(event)].keys():
                    await matcher.send(f"目标日志序列 {commands['remove']} 正在运行, 开始中止...")
                    loggers[get_group_id(event)][commands["remove"]][0].remove()
                    loggers[get_group_id(event)].pop(commands["remove"])

            Path(gl[commands["remove"]]).unlink()
            remove_logger(event, commands["remove"])
            return await matcher.send(f"日志序列 {commands['remove']} 已删除.")

        return await matcher.send("骰娘日志管理系统, 使用`.help log`指令详细信息.")

    def trpg_log(event) -> None:
        """ 外置的日志记录方法 """
        if not get_group_id(event) in loggers.keys():
            return

        for log in loggers[get_group_id(event)].keys():
            if isinstance(event, GroupMessageEvent):
                raw_json = json.loads(event.json())
                if raw_json['sender']['card']:
                    if raw_json['sender']['card'].lower() == "ob":
                        role_or_name = f"[旁观者 - {raw_json['sender']['nickname']}]"
                    elif raw_json['sender']['card'].lower() == "kp":
                        role_or_name = f"[主持人 - {raw_json['sender']['nickname']}]"
                    else:
                        role_or_name = f"[{raw_json['sender']['card']}]"
                elif raw_json['sender']['nickname']:
                    role_or_name = f"[未知访客 - {raw_json['sender']['nickname']}]"
                else:
                    role_or_name = f"[未知访客 - {str(event.get_user_id())}]"

                message = role_or_name + ": " + html.unescape(str(event.get_message()))
            elif isinstance(event, Event):
                message = f"[{get_name()}]: " + html.unescape(event.message)

            loggers[get_group_id(event)][log][0].info(message)

    @dgmessagemonitor.handle()
    @messagemonitor.handle()
    def loggerhandler(event: Event):
        """ 消息记录日志指令 """
        return trpg_log(event)

    @showcommand.handle()
    async def showhandler(matcher: Matcher, event: GroupMessageEvent, args: list=None):
        """ 角色卡展示指令 """
        if not get_status(event) and not event.to_me:
            return

        if not isinstance(args, list):
            args = format_msg(event.get_message(), begin=(".show", ".display"))
        else:
            args = args

        at = get_mentions(event)

        mode = get_mode(event)
        if mode in modes:
            try:
                sh = show_handler(event, args, at, mode=mode)
            except Exception as error:
                logger.exception(error)
                sh = [manager.process_generic_event(
                    "CommandFailed",
                    event=event
                )]
        else:
            return await matcher.send(manager.process_generic_event(
                "UnknownMode",
                event=event
            ))

        for msg in sh:
            await matcher.send(str(msg))

        return

    @setcommand.handle()
    async def sethandler(bot: V11Bot, matcher: Matcher, event: GroupMessageEvent):
        """ 角色卡设置指令 """
        if not get_status(event) and not event.to_me:
            return

        args = format_msg(event.get_message(), begin=(".set", ".st", ".s"), zh_en=True)
        at = get_mentions(event)
        commands = CommandParser(
            Commands([
                Only("show"),
                Only("del"),
                Only("clear"),
                Only("init"),
                Optional(("mode", "temp"), str),
                Optional(("name", "n"), str)
            ]),
            args,
            auto=True
        ).results

        if at and not get_user_card(event).upper() == "KP" and not event.to_me:
            await matcher.send(manager.process_generic_event(
                "SetPermissionDenied"
            ))
            return
        if len(at) == 1:
            qid = at[0]
        else:
            qid = ""

        mode = get_mode(event)

        if commands["name"]:
            cards: Cards = modes[mode].__cards__
            if not cards.get(event):
                cha: Character = modes[mode].__charactor__()
                if hasattr(cha, "init"):
                    cha.init()

                cards.update(event, cha.__dict__, save=True)

            cha: Character = modes[mode].__charactor__()
            cha.load(cards.get(event, qid=qid))
            setattr(cha, "name", commands["name"])
            await bot.set_group_card(group_id=event.group_id, user_id=event.user_id, card=commands["name"])
            return await matcher.send(f"命名角色为 {commands['name']}")

        if commands["init"]:
            cards: Cards = modes[mode].__cards__
            if not cards.get(event):
                cha: Character = modes[mode].__charactor__()
                if hasattr(cha, "init"):
                    cha.init()

                cards.update(event, cha.__dict__, save=True)

            return await matcher.send(
                manager.process_generic_event(
                    "CardInit",
                    event=event
                )
            )

        if commands["show"]:
            args.remove("show")
            return await showhandler(matcher, event, args=args)

        if commands["del"]:
            args.remove("del")
            return await delhandler(matcher, event, args=args)

        if commands["mode"]:
            if "mode" in args:
                args.remove("mode")
            else:
                args.remove("temp")
            return await modehandler(bot=bot, matcher=matcher, event=event, args=[commands["mode"]])

        if mode in modes:
            try:
                if not args:
                    cache = modes[mode].__cache__
                    user_id: int = event.user_id
                    got = cache.get(event, qid=str(user_id))

                    if isinstance(got, dict):
                        if "name" not in got.keys():
                            got = ""

                    name = got['name'] if got else ""
                    await bot.set_group_card(group_id=event.group_id, user_id=user_id, card=name)

                cards: Cards = modes[mode].__cards__
                if not cards.get(event):
                    cha: Character = modes[mode].__charactor__()
                    if hasattr(cha, "init"):
                        cha.init()

                    cards.update(event, cha.__dict__, save=True)

                sh = set_handler(event, args, at, mode=mode)
            except Exception as error:
                logger.exception(error)
                sh = [manager.process_generic_event(
                    "CommandFailed",
                    event=event
                )]
        else:
            return await matcher.send(manager.process_generic_event(
                "UnknownMode",
                event=event
            ))

        return await matcher.send(sh)

    @helpcommand.handle()
    async def helphandler(matcher: Matcher, event: MessageEvent):
        """ 帮助指令 """
        if not get_status(event) and not event.to_me:
            return

        args = format_msg(event.get_message(), begin=(".help", ".h"))
        if args:
            arg = args[0]
        else:
            arg = ""

        message = help_message(arg)
        message = re.sub(r"\{name\}", get_name(), message)
        message = re.sub(r"\{version\}", VERSION, message)
        message = re.sub(r"\{py_version\}", platform.python_version(), message)
        message = re.sub(r"\{nonebot_version\}", nonebot.__version__, message)
        return await matcher.send(message)

    @modecommand.handle()
    async def modehandler(bot: V11Bot, matcher: Matcher, event: MessageEvent, args: list=[]):
        """ 跑团模式切换指令 """
        if not get_status(event) and not event.to_me:
            return

        args = format_msg(event.get_message(), begin=(".mode", ".m")) if not args else args
        cp = CommandParser(
            Commands([
                Positional("mode", str),
            ]),
            args=args,
            auto=True
        )

        if not cp.nothing:
            cp = cp.results
            if cp["mode"] in modes:
                set_mode(event, cp["mode"])

                for user in await bot.get_group_member_list(group_id=event.group_id):
                    if not hasattr(modes[cp["mode"]], "__cards__"):
                        break

                    card: Cards = modes[cp["mode"]].__cards__
                    user_id: int = user['user_id']
                    got = card.get(event, qid=str(user_id))

                    if isinstance(got, dict):
                        if "name" not in got.keys():
                            continue
                    elif not got:
                        continue

                    name = got['name'] if got else ""
                    await bot.set_group_card(group_id=event.group_id, user_id=user_id, card=name)

                await matcher.send(manager.process_generic_event(
                    "ModeChanged",
                    event=event,
                    Mode=cp["mode"].upper()
                ))
                return True
            else:
                await matcher.send(manager.process_generic_event(
                    "UnknownMode",
                    event=event,
                    Mode=cp["mode"].upper()
                ))
                return True
        else:
            reply = "当前已正确安装的跑团插件:\n"
            for plugin in modes.keys():
                reply += f"{plugin.upper()} 模式: {plugin}.\n"

            reply += f"当前的跑团模式为 {get_mode(event).upper()}."
            await matcher.send(reply)

    @shootcommand.handle()
    async def shoothandler(matcher: Matcher, event: GroupMessageEvent):
        """ 射击检定指令 """
        if not get_status(event) and not event.to_me:
            return

        await matcher.send(shoot(event))

    @attackcommand.handle()
    async def attackhandler(matcher: Matcher, event: GroupMessageEvent):
        """ 伤害检定指令 """
        if not get_status(event) and not event.to_me:
            return

        args = format_str(event.get_message(), begin=(".at", ".attack"))
        mode = get_mode(event)
        if mode in modes:
            if not hasattr(modes[mode], "__commands__"):
                return await matcher.send(f"跑团模式 {mode.upper()} 未设置标准指令.")

            if not "at" in modes[mode].__commands__.keys():
                return await matcher.send(f"跑团模式 {mode.upper()} 不支持伤害检定指令.")

            handler = modes[mode].__commands__["at"]
            return await matcher.send(handler(event, args))
        else:
            return await matcher.send(manager.process_generic_event(
                "UnknownMode",
                event=event
            ))

    @damcommand.handle()
    async def damhandler(matcher: Matcher, event: GroupMessageEvent):
        """ 承伤检定指令 """
        if not get_status(event) and not event.to_me:
            return

        args = format_msg(event.get_message(), begin=(".dam", ".damage"))
        mode = get_mode(event)
        if mode in modes:
            if not hasattr(modes[mode], "__commands__"):
                return await matcher.send(f"跑团模式 {mode.upper()} 未设置标准指令.")

            if not "at" in modes[mode].__commands__.keys():
                return await matcher.send(f"跑团模式 {mode.upper()} 不支持承伤检定指令.")

            handler = modes[mode].__commands__["dam"]
            await matcher.send(handler(event, args))
        else:
            await matcher.send(manager.process_generic_event(
                "UnknownMode",
                event=event
            ))

    @encommand.handle()
    async def enhandler(matcher: Matcher, event: GroupMessageEvent):
        """ 属性或技能激励指令 """
        if not get_status(event) and not event.to_me:
            return

        args = format_msg(event.get_message(), begin=".en")
        mode = get_mode(event)
        if mode in modes:
            if not hasattr(modes[mode], "__commands__"):
                return await matcher.send(f"跑团模式 {mode.upper()} 未设置标准指令.")

            if not "at" in modes[mode].__commands__.keys():
                return await matcher.send(f"跑团模式 {mode.upper()} 不支持激励指令.")

            handler = modes[mode].__commands__["en"]
            await matcher.send(handler(event, args))
        else:
            await matcher.send(manager.process_generic_event(
                "UnknownMode",
                event=event
            ))

    @racommand.handle()
    async def rahandler(matcher: Matcher, event: GroupMessageEvent):
        """ 属性或技能检定指令 """
        if not get_status(event) and not event.to_me:
            return

        args = format_msg(event.get_message(), begin=".ra")
        mode = get_mode(event)
        if mode in modes:
            if not hasattr(modes[mode], "__commands__"):
                return await matcher.send(f"跑团模式 {mode.upper()} 未设置标准指令.")

            if not "ra" in modes[mode].__commands__.keys():
                return await matcher.send(f"跑团模式 {mode.upper()} 不支持技能检定指令.")

            handler = modes[mode].__commands__["ra"]
            replies = handler(event, args)
            if isinstance(replies, list):
                for reply in replies:
                    await matcher.send(reply)

                return

            await matcher.send(replies)
        else:
            await matcher.send(manager.process_generic_event(
                "UnknownMode",
                event=event
            ))

    @rhcommand.handle()
    async def rhhandler(bot: Bot, matcher: Matcher, event: GroupMessageEvent):
        """ 暗骰指令 """
        if not get_status(event) and not event.to_me:
            return

        args = format_str(event.get_message(), begin=".rh")
        await matcher.send("暗骰: 命运的骰子在滚动.")
        return await bot.send_private_msg(user_id=event.get_user_id(), message=roll(args))

    @rahcommand.handle()
    async def rhahandler(bot: Bot, matcher: Matcher, event: GroupMessageEvent):
        """ 暗骰技能检定指令 """
        if not get_status(event) and not event.to_me:
            return

        args = format_msg(event.get_message(), begin=".rah")
        mode = get_mode(event)
        if mode in modes:
            if not hasattr(modes[mode], "__commands__"):
                await matcher.send(f"跑团模式 {mode.upper()} 未设置标准指令.")
                return

            if not "ra" in modes[mode].__commands__.keys():
                await matcher.send(f"跑团模式 {mode.upper()} 不支持技能检定指令.")
                return

            await matcher.send("暗骰: 命运的骰子在滚动.")

            handler = modes[mode].__commands__["ra"]
            replies = handler(event, args)
            if isinstance(replies, list):
                for reply in replies:
                    await bot.send_private_msg(user_id=event.get_user_id(), message=reply)
                return

            await bot.send_private_msg(user_id=event.get_user_id(), message=replies)
        else:
            await matcher.send(manager.process_generic_event(
                "UnknownMode",
                event=event
            ))

    @rollcommand.handle()
    async def rollhandler(matcher: Matcher, event: MessageEvent):
        """ 标准掷骰指令 """
        if not get_status(event) and not event.to_me:
            return

        args = format_str(event.get_message(), begin=(".r", ".roll"))
        name = get_user_card(event)
        if not args:
            return await matcher.send(roll("1d100", name=name))

        try:
            await matcher.send(roll(args, name=name))
        except Exception as error:
            logger.exception(error)
            await matcher.send("未知错误, 可能是掷骰语法异常.\nBUG提交: https://gitee.com/unvisitor/issues")

    @delcommand.handle()
    async def delhandler(matcher: Matcher, event: GroupMessageEvent, args: list=None):
        """ 角色卡或角色卡技能删除指令 """
        if not get_status(event) and not event.to_me:
            return

        if not isinstance(args, list):
            args = format_msg(event.get_message(), begin=(".del", ".delete"))
        else:
            args = args

        at = get_mentions(event)

        if at and not is_super_user(event):
            await matcher.send(manager.process_generic_event(
                "DeletePermissionDenied",
                event=event
            ))
            return

        mode = get_mode(event)
        if mode in modes:
            for msg in del_handler(event, args, at, mode=mode):
                await matcher.send(msg)

    @rolecommand.handle()
    async def rolehandler(bot: V11Bot, matcher: Matcher, event: GroupMessageEvent):
        """ 身份组认证 """
        if not get_status(event) and not event.to_me:
            return

        args = format_msg(event.get_message(), begin=(".role"))
        cp = CommandParser(
            Commands([
                Only("kp"),
                Only("ob"),
                Only("exit"),
            ]),
            args=args,
            auto=True
        ).results

        if cp["kp"]:
            return await rolekphandler(bot, matcher, event)

        if cp["ob"]:
            return await roleobhandler(bot, matcher, event)

    @rolekpcommand.handle()
    async def rolekphandler(bot: V11Bot, matcher: Matcher, event: GroupMessageEvent):
        """ KP 身份组认证 """
        if not get_status(event) and not event.to_me:
            return

        args = format_msg(event.get_message(), begin=(".kp"))
        cp = CommandParser(
            Commands([
                Optional(("time", "schedule"), int),
                Optional(("minute", "min"), int, 0)
            ]),
            args=args,
            auto=True
        ).results

        if cp["time"]:
            @scheduler.scheduled_job("cron", hour=cp["time"], minute=cp["minute"], id="timer")
            async def _():
                await bot.send_group_msg(group_id=event.group_id, message="抵达 KP 设定的跑团启动时间.")

            try:
                scheduler.start()
            except:
                pass

            return await matcher.send(f"定时任务: {cp['time']}: {cp['minute'] if len(str(cp['minute'])) > 1 else '0'+str(cp['minute'])}")

        rolekp(event)
        await bot.set_group_card(group_id=event.group_id, user_id=event.get_user_id(), card="KP")
        return await matcher.send(
            manager.process_generic_event(
                "RoleKP",
                event=event
            )
        )

    @roleobcommand.handle()
    async def roleobhandler(bot: V11Bot, matcher: Matcher, event: GroupMessageEvent):
        """ OB 身份组认证 """
        if not get_status(event) and not event.to_me:
            return

        import json
        roleob(event)

        if json.loads(event.json())['sender']['card'] == "ob":
            await bot.set_group_card(group_id=event.group_id, user_id=event.get_user_id())
            return await matcher.send("取消旁观者 (OB) 身份.")
        else:
            await bot.set_group_card(group_id=event.group_id, user_id=event.get_user_id(), card="ob")
            return await matcher.send(
                manager.process_generic_event(
                    "RoleOB",
                    event=event
                )
            )

    @registcommand.handle()
    async def registhandler(matcher: Matcher, event: GroupMessageEvent):
        """ 消息事件注册指令 """
        if not get_status(event) and not event.to_me:
            return

        args = format_str(event.get_message(), begin=(".regist", ".reg"), lower=False).split(" ")
        args = list(filter(None, args))
        cp = CommandParser(
            Commands([
                Positional("event_name", str),
                Positional("message", str),
                Optional(("remove", "rm", "delete", "del"), str),
                Optional("enable", str),
                Optional("disable", str),
            ]),
            args=args,
            auto=True
        ).results

        if cp["remove"]:
            if not manager.remove_event(cp["remove"]):
                return await matcher.send("错误的消息类型, 使用`.help regist`获取帮助信息.")

            return await matcher.send(f"消息事件 {cp['remove']} 已经成功销毁, 该事件将采用默认回复替代.")

        if cp["enable"]:
            if not manager.enable_event(cp["enable"]):
                matcher.send("启用消息事件时出现异常, 请检查事件名是否正确!\n使用`.help regist`获得帮助信息.")\

            return await matcher.send(f"消息事件 {cp['enable']} 已启用.")

        if cp["disable"]:
            if not manager.enable_event(cp["disable"]):
                matcher.send("禁用消息事件时出现异常, 请检查事件名是否正确!\n使用`.help regist`获得帮助信息.")\

            return await matcher.send(f"消息事件 {cp['disable']} 已启用.")

        event_name = cp["event_name"]
        message = cp["message"]
        if not event_name or not message:
            return await matcher.send("消息事件注册参数不全, 使用`.help regist`获取帮助信息.")

        message = html.unescape(message)
        manager.register_event(event_name, message.replace("\\n", "\n"), is_custom=True)
        return await matcher.send(f"消息事件 {event_name} 已被更改为 {message}.")

    @versioncommand.handle()
    async def versionhandler(matcher: Matcher, event: GroupMessageEvent):
        """ 骰娘版本及开源声明指令 """
        return await matcher.send(f"Unvisitor DicerGirl 版本 {VERSION} [Python {platform.python_version()} For Nonebot2 {nonebot.__version__}]\n此项目以Apache-2.0协议开源.\nThis project is open source under the Apache-2.0 license.\n欢迎使用 DicerGirl, 使用`.help 指令`查看指令帮助.")
else:
    logger.warning("Nonebot2 初始化失败, DicerGirl 无法启动!")


import re
import string
from enum import Enum, unique


@unique
class MatchType(Enum):
    """
    EXACT_MATCH 完全匹配
    PARTIAL_MATCH 部分匹配
    REGEX_MATCH 正则匹配
    FUNCTION_MATCH 方法匹配
    """
    EXACT_MATCH = 0
    PARTIAL_MATCH = 1
    REGEX_MATCH = 2
    FUNCTION_MATCH = 3


class TextMatcher:
    def match(self, send_text, match_field: string, match_type: MatchType) -> bool:
        """
        匹配方法
        """
        if match_type == MatchType.EXACT_MATCH:
            return self.__exact_matcher(send_text, match_field)
        elif match_type == MatchType.PARTIAL_MATCH:
            tmp = match_field.split(";")
            return self.__partial_matcher(send_text, *tmp)
        elif match_type == MatchType.REGEX_MATCH:
            return self.__regex_matcher(send_text, match_field)
        elif match_type == MatchType.FUNCTION_MATCH:
            # TODO
            pass

    @staticmethod
    def __exact_matcher(first, second) -> bool:
        """
         完全匹配器
        """
        return str(first) == str(second)

    @staticmethod
    def __partial_matcher(text, *args) -> bool:
        """
        部分匹配器
        """
        for arg in args:
            if str(arg) in str(text):
                return True
        return False

    @staticmethod
    def __regex_matcher(text, regex) -> bool:
        """
        正则匹配器
        """
        return bool(re.match(regex, str(text)))


matcher = TextMatcher()



from dicergirl.utils.dicer import Dicer
from dicergirl.utils.docimasy import expr
from dicergirl.utils.cards import Cards
from typing import Any, Dict, List

class DefaultCommand:
    def __init__(self, name: str, cards: Cards, cache: Cards, charactor, attrs_dict: Dict[str, List[str]]) -> None:
        self.name = name
        self.cards = cards
        self.cache = cache
        self.charactor = charactor
        self.attrs_dict = attrs_dict

class DefaultRA(DefaultCommand):
    def __init__(self, name: str, cards: Cards, cache: Cards, charactor, attrs_dict: Dict[str, List[str]]) -> None:
        super().__init__(name, cards, cache, charactor, attrs_dict)

    def __call__(self, event, args: list) -> str:
        if len(args) == 0:
            return "你在整什么活? 检定技能需要给入技能名称.\n使用`.help ra`指令查看指令使用方法."
        if len(args) > 2:
            return "给入参数过多(最多2需要但%d给予)." % len(args)
        
        card_data = self.cards.get(event)
        if not card_data:
            if len(args) == 1:
                return f"你尚未保存人物卡, 请先执行`.{self.name}`车卡并执行`.set`保存.\n如果你希望快速检定, 请执行`.ra [str: 技能名] [int: 技能值]`."

            return str(expr(Dicer(), int(args[1])))
        
        inv = self.charactor().load(card_data)

        is_base = False
        exp = None
        for _, alias in self.attrs_dict.items():
            if args[0] in alias:
                exp = int(getattr(inv, alias[0]))
                is_base = True
                break

        if not is_base:
            for skill in inv.skills:
                if args[0] == skill:
                    exp = inv.skills[skill]
                    break
                else:
                    exp = False

        if not exp:
            if len(args) == 1:
                return "你没有这个技能, 如果你希望快速检定, 请执行`.ra [str: 技能名] [int: 技能值]`."

            if not args[1].isdigit():
                return "技能值应当为整型数, 使用`.help ra`查看技能检定指令使用帮助."

            return str(expr(Dicer(), int(args[1])))
        elif exp and len(args) > 1:
            if not args[1].isdigit():
                return "技能值应当为整型数, 使用`.help ra`查看技能检定指令使用帮助."

            reply = [f"你已经设置了技能 {args[0]} 为 {exp}, 但你指定了检定值, 使用指定检定值作为替代."]
            reply.append(str(expr(Dicer(), int(args[1]))))
            return reply

        time = 1
        r = expr(Dicer(), exp)

        for _ in range(time-1):
            r += expr(Dicer(), exp)

        return r.detail

class DefaultAT(DefaultCommand):
    def __init__(self, roll: str="1d6") -> None:
        self.roll = roll

    def __call__(self, event, args) -> str:
        rd = Dicer(self.roll).roll()
        return f"进行伤害检定:\n{rd.description()}\n造成了 {rd.outcome} 点伤害!"

class DefaultDAM(DefaultCommand):
    def __init__(self, roll: str="1d6") -> None:
        self.roll = roll

    def __call__(self, event, args) -> str:
        rd = Dicer(self.roll).roll()
        return f"进行承伤检定:\n{rd.description()}\n受到了 {rd.outcome} 点伤害!"


from dicergirl.reply.parsers.matcher import MatchType


class GenericResponse:
    """
    存储 DicerGirl 或 DicerGirl 插件的注册名及其回复文本

    :argument event_name 事件名
    :argument send_text 事件对应的响应文本
    :argument enable 是否启用响应事件
    """

    def __init__(self, event_name: str, send_text: str, enable=True):
        self.event_name = event_name
        self.send_text = send_text
        self.enable = enable

    def to_dict(self):
        return {'enable': self.enable, 'send_text': self.send_text}


class ConditionResponse(GenericResponse):
    """
    存储用户指定条件(正则/字符串/函数体)及符合条件的回复文本

    :argument match_field 匹配字段，例如: "<[^>]*>"
    :argument match_type 匹配类型:相等、包含、正则、特定方法
    :argument enable 是否启用响应事件
    """

    def __init__(self, event_name: str,
                 send_text: str,
                 match_field: str,
                 match_type: MatchType,
                 enable: bool = True):
        super().__init__(event_name, send_text, enable)
        self.match_type = match_type
        self.match_field = match_field

    def to_dict(self):
        return {'enable': self.enable, 'send_text': self.send_text, 'match_field': self.match_field,
                'match_type': self.match_type.name}



import abc

from ..common.const import EMPTY_DICT
from typing import Literal

class Character:
    """ 人物卡模板 """
    name: str
    sex: Literal["男", "女"]
    age: int
    hp: int
    hp_max: int

    def __init__(self) -> None:
        self.skills = EMPTY_DICT
        self.tools = EMPTY_DICT

    @abc.abstractmethod
    def init(self):
        raise NotImplementedError

    def __repr__(self):
        data = "姓名: %s\n" % self.name
        data += "性别: %s 年龄: %d\n" % (self.sex, self.age)
        data += "生命值: %d/%d" % (self.hp, self.hp_max)
        return data

    def skills_output(self) -> str:
        if not self.skills:
            return "%s 当前无任何技能数据。" % self.name
        r = "%s技能数据: " % self.name
        for k, v in self.skills.items():
            r += "\n%s: %d" % (k, v)
        return r

    def output(self) -> str:
        return self.__repr__()

    def rollcount(self) -> tuple:
        return (self.__count(), self.__count() + self.luc)

    def __count(self):
        raise NotImplementedError

    def load(self, data: dict):
        self.__dict__.update(data)
        return self


from typing import Dict

from multilogging import multilogger

from dicergirl.common.response import GenericResponse, ConditionResponse
from dicergirl.reply.parsers.matcher import matcher

logger = multilogger(name="DicerGirl", payload="ReplyData")


class GenericData:
    def __init__(self,
                 group_name: str,
                 version: str,
                 author: str = "",
                 description: str = "",
                 enable: bool = True):
        self.group_name = group_name
        self.items: Dict = {}
        self.version = version
        self.author = author
        self.description = description
        self._enable = enable

    def add(self, *args):
        for data in args:
            if isinstance(data, GenericResponse):
                self.items[data.event_name] = data

    def remove(self, *args):
        for arg in args:
            if isinstance(arg, GenericResponse):
                del self.items[arg.event_name]
            elif isinstance(arg, str):
                if self.items.get(arg) is not None:
                    self.items.pop(arg)

    def enable(self, event_name: str = None):
        return self.set_event_status(event_name, True)

    def disable(self, event_name: str = None):
        return self.set_event_status(event_name, False)

    def toggle(self, event_name: str = None):
        return self.set_event_status(event_name)

    def set_event_status(self, event_name: str = None, status: bool = None):
        if event_name is None:
            self._enable = not self._enable if not status else status
        elif event_name in self.items:
            event = self.items[event_name]
            event.enable = status if status is not None else not event.enable
            return event  # Add this line to return the updated event
        return None

    def get_response(self, event_name: str) -> GenericResponse:
        for key, value in self.items.items():
            if key == event_name:
                return value

    def is_enable(self, event_name: str = None):
        return self._enable and (not event_name or (self.items.get(event_name) and self.items[event_name].enable))


class ConditionData(GenericData):
    def __init__(self, group_name: str,
                 version: str,
                 author: str = "",
                 description: str = "",
                 enable: bool = True):
        super().__init__(group_name, version, author, description, enable)

    def add(self, *args):
        for data in args:
            if isinstance(data, ConditionResponse):
                self.items[data.event_name] = data

    def get_responses(self, message: str):
        return_dict = {}
        for value in self.items.values():
            if isinstance(value, ConditionResponse):
                if matcher.match(message, value.match_field, value.match_type):
                    return_dict[self] = value
        return return_dict



from pathlib import Path
from typing import Dict, List
from dicergirl.common.const import BOT_MODES_FILE, BOT_STATUS_FILE, DICERGIRL_DATA_PATH, DICERGIRL_LOGS_PATH, LOGGERS_CACHE_FILE, SAVED_DATA_PATH, SUPERUSER_FILE
from dicergirl.reply.init import init as reply_init
from dicergirl.utils.handlers import get_group_id
from dicergirl.utils.loggers import load_loggers
from dicergirl.utils.settings import change_status, load_status_settings
from multilogging import multilogger

import json


logger = multilogger(name="DicerGirl", payload="utils.operator")
""" `operator.py`日志 """


def set_name(name) -> bool | str:
    """ 给骰娘命名 """
    if len(name) >= 5:
        return "我不想要太长的名字!"

    with open(DICERGIRL_DATA_PATH / "name", "w") as f:
        f.write(name)

    return True


def get_name() -> str:
    """ 获得骰娘的名字 """
    path = DICERGIRL_DATA_PATH / "name"
    if not path.exists():
        return "欧若可"

    return path.open(mode="r").read()


def botoff(event):
    """ 机器人在`event`所指向的群聊中开启指令限制 """
    status = load_status_settings()
    status[get_group_id(event)] = False
    change_status(status)
    f = open(BOT_STATUS_FILE, "w")
    json.dump(status, f)


def boton(event):
    """ 机器人在`event`所指向的群聊中开启完全功能 """
    status = load_status_settings()
    status[get_group_id(event)] = True
    change_status(status)
    f = open(BOT_STATUS_FILE, "w")
    json.dump(status, f)


def get_status(event):
    """ 判断机器人在`event`所指向的群聊中是否处于完全功能状态 """
    status = load_status_settings()
    group_id = get_group_id(event)

    if group_id in ("private", "botmessage"):
        return True

    if group_id not in status.keys():
        status[get_group_id(event)] = True
        f = open(BOT_STATUS_FILE, "w")
        json.dump(status, f)
        return True

    return status[get_group_id(event)]


def load_status() -> dict:
    """ 导入目前所存储的机器人在各群聊中状态 """
    status_text = BOT_STATUS_FILE.read_text(encoding="utf-8")
    if status_text:
        status = json.loads(status_text)
    else:
        status = {}

    change_status(status)
    return status


def init() -> None:
    """ 骰娘初始化 """
    global saved_loggers
    dirs: Dict[str, List[Path, list]] = {
        "Dicer Girl": [DICERGIRL_DATA_PATH, "dir"],
        "Dicer Girl 数据": [SAVED_DATA_PATH, "dir"],
        "Dicer Girl 日志": [DICERGIRL_LOGS_PATH, "dir"],
        "Dicer Girl 状态管理": [BOT_STATUS_FILE, "file"],
        "日志管理": [LOGGERS_CACHE_FILE, "file"],
        "跑团模式存储": [BOT_MODES_FILE, "file"],
        "超级用户存储": [SUPERUSER_FILE, "file"]
    }
    for name, dir in dirs.items():
        if not dir[0].exists():
            logger.info(f"{name}{'文件夹' if dir[1] == 'dir' else '文件'}未建立, 建立它.")
            if dir[1] == "dir":
                dir[0].mkdir(parents=True, exist_ok=True)
            else:
                with open(dir[0], "w", encoding="utf-8") as f:
                    f.write("{}")
    saved_loggers = load_loggers()
    load_status()
    reply_init()


import asyncio
import httpx


async def get_latest_version(package_name):
    """ 获取当前 Pypi 上`dicergirl`的最新版本号 """
    async with httpx.AsyncClient() as client:
        url = f"https://pypi.org/pypi/{package_name}/json"
        try:
            response = await client.get(url)
        except httpx.ReadTimeout:
            return "0.0.0"

        if response.status_code == 404:
            return "0.0.0"

        package_info = response.json()
        return package_info["info"]["version"]


async def run_shell_command(command):
    """ 异步执行 shell 指令的原始方法 """
    process = await asyncio.create_subprocess_shell(
        command,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.PIPE
    )

    stdout, stderr = await process.communicate()

    return {
        "stdout": stdout.decode().strip(),
        "stderr": stderr.decode().strip(),
        "returncode": process.returncode
    }


from typing import Dict, Any
from pathlib import Path
from multilogging import multilogger
from yaml.loader import FullLoader
from nonebot.adapters import Event

from .handlers import get_user_id, get_group_id
from ..common.const import SAVED_DATA_PATH

import yaml


logger = multilogger(name="DicerGirl", payload="utils.cards")
""" `utils.cards`日志 """


class Cards:
    """ DicerGirl 人物卡数据操作类 """
    data_path: Path = SAVED_DATA_PATH

    def __init__(self, mode: str=None, cache_path: Path=None):
        self.data: Dict[str, Dict[str, Any]] = {}
        self.mode = mode if mode else "未知模式"
        self.cache_path = cache_path if cache_path else self.data_path / f"{mode}_cards.yaml"

    def init(self):
        logger.info(f"{self.mode.upper()} 存储文件未建立, 建立它.")
        if not self.cache_path.parent.exists():
            self.cache_path.parent.mkdir(parents=True, exist_ok=True)

        with self.cache_path.open(mode="w", encoding="utf-8") as f:
            yaml.dump({"mode": self.mode, "cards": {}}, f)

    def save(self):
        if not self.cache_path.exists():
            self.init()

        with self.cache_path.open(mode="w", encoding="utf-8") as f:
            yaml.dump({"mode": self.mode, "cards": self.data}, f, allow_unicode=True)

    def load(self) -> dict:
        if not self.cache_path.exists():
            self.init()

        with self.cache_path.open(mode="r", encoding="utf-8") as f:
            data = yaml.load(f, FullLoader)
            if data is None:
                self.data = {}
            else:
                self.data = data["cards"]

        return self.data

    def update(self, event: Event, cha_dict: dict, qid: str="", save: bool=True) -> None:
        group_id = get_group_id(event)
        if not self.data.get(group_id):
            self.data[group_id] = {}

        self.data[group_id].update(
            {qid if qid else get_user_id(event): cha_dict}
            )

        return self.save() if save else None

    def get(self, event: Event, qid=""):
        group_id = get_group_id(event)
        if self.data.get(group_id):
            if self.data[group_id].get(qid if qid else get_user_id(event)):
                return self.data[group_id].get(qid if qid else get_user_id(event))
        else:
            return None

    def delete(self, event: Event, qid: str = "", save: bool = True) -> bool:
        if self.get(event, qid=qid):
            if self.data[get_group_id(event)].get(qid if qid else get_user_id(event)):
                self.data[get_group_id(event)].pop(
                    qid if qid else get_user_id(event))
            if save:
                self.save()
            return True
        return False

    def delete_skill(self, event: Event, skill_name: str, qid: str = "", save: bool = True) -> bool:
        if self.get(event, qid=qid):
            data = self.get(event, qid=qid)
            if data["skills"].get(skill_name):
                data["skills"].pop(skill_name)
                self.update(event, data, qid=qid, save=save)
                return True
        return False


from typing import Dict, Literal
from multilogging import multilogger

logger = multilogger(name="Dicer Girl", payload="Settings")
""" `settings.py`日志管理系统 """
status: Dict[str, bool] = {}
""" 机器人在各个群聊状态 """
DEBUG = False
""" 漏洞监测模式 """

def change_status(var) -> bool:
    """ 录入新的`status`内容 """
    global status
    status = var
    return True

def load_status_settings() -> bool:
    """ 导出当前机器人在各群聊的状态 """
    return status

def is_debug() -> Literal[False]:
    return DEBUG

def debugon() -> Literal[True]:
    global DEBUG
    DEBUG = True

def debugoff() -> Literal[True]:
    global DEBUG
    DEBUG = False


"""
初始化DGI_PROVIDERS与CUSTOM_PROVIDERS数组
"""
import os
import re

from multilogging import multilogger

from dicergirl.common import const
from dicergirl.common.response import GenericResponse, ConditionResponse
from dicergirl.reply.data import GenericData, ConditionData
from dicergirl.reply.manager import manager
from dicergirl.reply.parsers import templates
from dicergirl.reply.parsers.matcher import MatchType

logger = multilogger(name="DicerGirl", payload="ReplyModuleInit")


def init():
    """
    初始化方法
    """
    if not os.path.exists(const.REPLY_FOLDER_PATH):
        os.makedirs(const.REPLY_FOLDER_PATH)
    _load_template_methods()
    _init_default_template_file()
    _init_reply_config()


def _load_template_methods():
    """
    获取 templates.py 中的所有方法
    """
    for name, method in vars(templates).items():
        if callable(method):
            manager.register_method(method)


def _init_reply_config():
    """
    加载reply文件数组中
    """
    try:
        for filename in os.listdir(const.REPLY_FOLDER_PATH):
            filepath = os.path.join(const.REPLY_FOLDER_PATH, filename)
            # logger.info(f"载入文件: [{filepath}]")
            if os.path.isfile(filepath):
                is_execute = _load_custom_generic_reply_file(filename, filepath)
                if not is_execute:
                    _load_condition_specific_reply_file(filename, filepath)
    except Exception as e:
        logger.error(f"Error: {e}")


def _load_reply_file(filename, filepath, cache, container_class) -> bool:
    try:
        with open(filepath, "rb") as file:
            data = const.REPLY_YAML.load(file)

            group_name = filename.removesuffix(".yml")
            version = data["version"]
            author = data["author"]
            description = data["description"]
            enable = data["enable"]
            container = container_class(group_name, version, author, description, enable)
            items = data["items"]
            cache[filepath] = data
            for item in items:
                for event_name, event_content in item.items():
                    if isinstance(container, ConditionData):
                        container.add(ConditionResponse(
                            event_name,
                            event_content["send_text"],
                            event_content["match_field"],
                            MatchType[event_content["match_type"]],
                            event_content["enable"]
                        ))
                    elif isinstance(container, GenericData):
                        container.add(GenericResponse(event_name, event_content["send_text"], event_content["enable"]))
            return manager.register_container(container)
    except KeyError as e:
        logger.error(
            f"请确保您的回复配置文件包含了正确的键和相应的值。如果您不确定如何正确配置文件，请参考文档或向管理员寻求帮助。")
        logger.error(f"Error: {e}")
    except Exception as e:
        logger.error(f"Error: {e}")


def _load_custom_generic_reply_file(filename, filepath):
    if re.match(r'^dg-.*\.yml$', filename):
        return _load_reply_file(filename, filepath, const.GENERIC_REPLY_FILE_CACHE, GenericData)
    return False


def _load_condition_specific_reply_file(filename, filepath):
    if filename.endswith(".yml"):
        return _load_reply_file(filename, filepath, const.CONDITION_SPECIFIC_REPLY_FILE_CACHE, ConditionData)
    return False


def _init_default_template_file():
    """
    默认模板文件初始化
    """
    init_yaml_file(const.GENERIC_REPLY_FILE_PATH, const.CUSTOM_GENERIC_TEMPLATE)
    init_yaml_file(const.CONDITION_SPECIFIC_REPLY_FILE_PATH, const.CONDITION_SPECIFIC_TEMPLATE)
    init_yaml_file(const.EXAMPLE_CONDITION_SPECIFIC_REPLY_FILE_PATH, const.EXAMPLE_CONDITION_SPECIFIC_TEMPLATE)


def init_yaml_file(filepath, yaml_template):
    try:
        if not os.path.exists(filepath):
            with open(file=filepath, mode='wb') as drf:
                raw_data = const.REPLY_YAML.load(yaml_template)
                const.REPLY_YAML.dump(data=raw_data, stream=drf)
    except Exception as e:
        logger.error(f"初始化回复配置时出现异常。")
        logger.error(f"Error: {e}")



from typing import List
from multilogging import multilogger

import re


logger = multilogger(name="DicerGirl", payload="utils.cards")
""" `utils.cards`日志 """


def translate_punctuation(string) -> str:
    """ 中文字符转换为英文字符 """
    punctuation_mapping = {
        '，': ',',
        '。': '.',
        '！': '!',
        '？': '?',
        '；': ';',
        '：': ':',
        '“': '"',
        '”': '"',
        '‘': "'",
        '’': "'",
        '（': '(',
        '）': ')',
        '【': '[',
        '】': ']',
        '《': '<',
        '》': '>',
    }
    for ch_punct, en_punct in punctuation_mapping.items():
        string = string.replace(ch_punct, en_punct)
    return string


def format_str(message: str, begin=None, lower=True) -> str:
    """ 骰娘指令转义及解析 """
    regex = r"[<\[](.*?)[\]>]"
    message = str(message).lower() if lower else str(message)
    msg = re.sub("\s+", " ", re.sub(regex, "", message)).strip(" ")
    msg = translate_punctuation(msg)
    logger.debug(msg)

    if begin:
        if isinstance(begin, str):
            begin = [begin, ]
        elif isinstance(begin, tuple):
            begin = list(begin)

        begin.sort(reverse=True)
        for b in begin:
            msg = msg.replace(b, "").lstrip(" ")

    logger.debug(msg)
    return msg


def format_msg(message, begin=None, zh_en=True) -> List[str]:
    """ 骰娘指令拆析为`list`的方法 """
    msgs = format_str(message, begin=begin)
    outer = []
    regex = r'([+-]?\d+)|("[^"]+")|([a-zA-Z\u4e00-\u9fa5]+)' if not zh_en else r'([+-]?\d+)|([a-zA-Z]+)|("[^"]+")|([\u4e00-\u9fa5]+)'
    msgs = list(filter(None, re.split(regex, msgs)))
    logger.debug(msgs)

    for msg in msgs:
        splited_msg = list(filter(None, re.split(regex, msg.strip(" "))))

        for i, msg in enumerate(splited_msg):
            splited_msg[i] = msg.strip('"')

        outer += splited_msg

    msgs = list(filter(None, outer))
    logger.debug(msgs)
    return msgs


from nonebot.adapters.onebot.v11 import GroupMessageEvent, PrivateMessageEvent
from multilogging import multilogger
from typing import Callable, List

import inspect
import json


logger = multilogger(name="DicerGirl", payload="utils.handlers")
""" `utils.handlers`日志 """


def get_mentions(event: GroupMessageEvent) -> List[str]:
    """ 获取`event`指向的消息所有被`@`的用户 QQ 号 """
    mentions = []
    message = json.loads(event.json())["message"]

    for mention in message:
        if mention["type"] == "at":
            mentions.append(mention["data"]["qq"])

    return mentions


def get_handlers(main) -> List[Callable]:
    """ 获取目前所有的指令触发函数方法 """
    commands_functions = []

    for _, obj in vars(main).items():
        if inspect.isfunction(obj) and hasattr(obj, '__annotations__'):
            annotations = obj.__annotations__
            if annotations.get('message') is GroupMessageEvent:
                commands_functions.append(obj)

    return commands_functions


def get_group_id(event) -> str:
    """ 获取`event`指向的群聊`ID` """
    try:
        if isinstance(event, PrivateMessageEvent):
            return "private"
        elif isinstance(event, GroupMessageEvent):
            return str(event.group_id)
        else:
            if hasattr(event, "post_type"):
                if event.post_type == "message_sent":
                    return "botmessage"
            return "0"
    except Exception as error:
        logger.exception(error)
        return "0"


def get_user_id(event) -> str:
    """ 获取`event`指向的用户`ID` """
    try:
        return str(event.user_id)
    except Exception as error:
        logger.exception(error)
        return "0"


def get_user_card(event) -> str:
    """ 获取`event`指向的用户群名片 """
    try:
        raw_json = json.loads(event.json())['sender']
        if raw_json['card']:
            return raw_json['card']
        else:
            return raw_json['nickname']
    except:
        return "未知用户"


def get_user_nickname(event) -> str:
    """ 获取用户昵称 """
    try:
        raw_json = json.loads(event.json())['sender']
        return raw_json['nickname']
    except:
        return "未知用户"


async def get_friend_qids(bot) -> List[str]:
    result = []
    friends = await bot.get_friend_list()
    for friend in friends:
        result.append(friend["user_id"])

    return result


from dicergirl.common.const import VERSION

import setuptools

with open("README.md", encoding="utf-8") as f:
    long_description = f.read()

setuptools.setup(
    name = "dicergirl",
    version = VERSION,
    author = "Night Resurgent <fu050409@163.com>",
    author_email = "fu050409@163.com",
    description = "新一代跨平台开源 TRPG 骰娘框架",
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = "https://gitee.com/unvisitor/dicer",
    project_urls = {
        "Bug Tracker": "https://gitee.com/unvisitor/dicer/issues",
    },
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    license = "Apache-2.0",
    packages = setuptools.find_packages(),
    install_requires = [
        'nonebot2',
        'nonebot-adapter-onebot',
        'loguru',
        'pyyaml',
        'openai',
        'multilogging',
        'psutil',
        'nonebot_plugin_apscheduler',
        'httpx',
        'ruamel.yaml',
        'dicergirl-plugin-coc',
        'python-Levenshtein',
    ],
    python_requires=">=3",
)


from typing import Type
from nonebot.consts import STARTSWITH_KEY
from nonebot.matcher import Matcher
from nonebot.adapters import Event
from nonebot.plugin import on_message
from nonebot.rule import Rule

from ..utils.formatters import translate_punctuation

import re


class StartswithRule:
    """
    自定义的指令检查方法
    允许:
        1. 无视中英文字符串
        2. 无视前后多余空字符
    """
    __slots__ = ("msg", "ignorecase")

    def __init__(self, msg, ignorecase=False):
        self.msg = msg
        self.ignorecase = ignorecase

    def __repr__(self) -> str:
        return f"Startswith(msg={self.msg}, ignorecase={self.ignorecase})"

    def __eq__(self, other: object) -> bool:
        return (
            isinstance(other, StartswithRule)
            and frozenset(self.msg) == frozenset(other.msg)
            and self.ignorecase == other.ignorecase
        )

    def __hash__(self) -> int:
        return hash((frozenset(self.msg), self.ignorecase))

    async def __call__(self, event, state) -> bool:
        try:
            text = translate_punctuation(event.get_plaintext()).strip()
        except Exception:
            return False
        if match := re.match(
            f"^(?:{'|'.join(re.escape(prefix) for prefix in self.msg)})",
            text,
            re.IGNORECASE if self.ignorecase else 0,
        ):
            state[STARTSWITH_KEY] = match.group()
            return True
        return False


def startswith(msg, ignorecase=True) -> Rule:
    """ 实例化指令检查方法 """
    if isinstance(msg, str):
        msg = (msg,)

    return Rule(StartswithRule(msg, ignorecase))


def on_startswith(commands, priority=0, block=True) -> Type[Matcher]:
    """ 获得`Nonebot2`指令检查及参数注入方法 """
    if isinstance(commands, str):
        commands = (commands, )

    return on_message(startswith(commands, True), priority=priority, block=block, _depth=1)


class PluginNotFoundError(FileNotFoundError):
    """ 插件不存在 """

class PluginExistsError(FileExistsError):
    """ 插件已安装 """

class PluginInstallFailedError(RuntimeError):
    """ 插件安装失败 """

class PluginUninstallFailedError(RuntimeError):
    """ 插件卸载失败 """